//S88 control board

// reference : RB_S88
// Software by Ruud Boer, November 2014.
// https://rudysmodelrailway.wordpress.com/software/

//151101 fujigaya2  6ch photo detector version 
//151114 fujigaya2 (111507 board debug)
//151121 fujigaya2 when one of the sensors is on  ,RUN LED is on.
//151230 fujigaya2 change original source
//180316 8ch ver for Arduino nano

int clk_counter=0;
const byte dataIn=0;  //data input from next Arduino in S88 chain
const byte dataOut=1; //data output pin=6

//Sensor control
#define RUN_LED_PIN 13 // RUN_LED(status LED) on/off pin
#define SENSOR_INTERVAL 50 //sensor interval[ms]
#define ANALOG_PORT_NUM 8 //analog port number (arduino uno'number is 6)
#define CONT_PORT_NUM 16 //port_num to see from DCC Controller

//Pin 関係のdefine
//IR LED output port
#define IR_LED_PIN0  12 // IR_LED(sensor LED) on/off pin
#define IR_LED_PIN1  11 // IR_LED(sensor LED) on/off pin
#define IR_LED_PIN2  10 // IR_LED(sensor LED) on/off pin
#define IR_LED_PIN3  9 // IR_LED(sensor LED) on/off pin
#define IR_LED_PIN4  8 // IR_LED(sensor LED) on/off pin
#define IR_LED_PIN5  7 // IR_LED(sensor LED) on/off pin
#define IR_LED_PIN6  6 // IR_LED(sensor LED) on/off pin
#define IR_LED_PIN7  5 // IR_LED(sensor LED) on/off pin
//Analog input port
#define ANALOG_PIN0  0 // analog pin number
#define ANALOG_PIN1  1 // analog pin number
#define ANALOG_PIN2  2 // analog pin number
#define ANALOG_PIN3  3 // analog pin number
#define ANALOG_PIN4  4 // analog pin number
#define ANALOG_PIN5  5 // analog pin number
#define ANALOG_PIN6  6 // analog pin number
#define ANALOG_PIN7  7 // analog pin number

int led_pin[ANALOG_PORT_NUM] = {
  IR_LED_PIN0,
  IR_LED_PIN1,
  IR_LED_PIN2,
  IR_LED_PIN3,
  IR_LED_PIN4,
  IR_LED_PIN5,
  IR_LED_PIN6,
  IR_LED_PIN7
  };
  
int analog_pin[ANALOG_PORT_NUM] = {
  ANALOG_PIN0,
  ANALOG_PIN1,
  ANALOG_PIN2,
  ANALOG_PIN3,
  ANALOG_PIN4,
  ANALOG_PIN5,
  ANALOG_PIN6,
  ANALOG_PIN7
  };

unsigned long gPreviousL5 = 0; 
boolean ir_led_onoff =false;
int reflector_state = 0;
int light_on_value[ANALOG_PORT_NUM];
int light_off_value[ANALOG_PORT_NUM];
int diff_value[ANALOG_PORT_NUM];
int transfer_value[CONT_PORT_NUM];
int sensor_value[CONT_PORT_NUM]; 

int diff_value_threshold = 50;

void setup() {
  //S88 Control
  pinMode(2, INPUT_PULLUP);
  attachInterrupt(0,clk,RISING); //pin 2 = clock interrupt
  pinMode(3, INPUT_PULLUP);
  attachInterrupt(1,PS,RISING);    //pin 3 = PS interrupt
  pinMode(dataIn,INPUT_PULLUP); // data in from next Arduino S88 in chain
  pinMode(dataOut, OUTPUT); // data out to commnad station or to previous Arduino in S88 chain
  digitalWrite(dataOut, LOW);   
  
  //Sensor Control
  for(int i = 0;i < ANALOG_PORT_NUM;i++)
  {
    pinMode(led_pin[i], OUTPUT);
  }
  //Status LED Control
  pinMode(RUN_LED_PIN, OUTPUT);
  gPreviousL5 = millis();
  reflector_state = 0;
  //Serial.begin(115200);
  
  //init sensor_value,transfer_value
  for(int i = 0; i < CONT_PORT_NUM;i++)
  {
    transfer_value[i] = 0;
    sensor_value[i] = 0;
  }
}

void loop()
{
  if(millis() - gPreviousL5 > SENSOR_INTERVAL)
  {
    gPreviousL5 = millis();
    detect_analog();
  }
}

void PS()
{
  //Set transfer value
  clk_counter=0;
  for(int i = 0 ;i < CONT_PORT_NUM ; i++)
  {
    transfer_value[i] = sensor_value[i];
    sensor_value[i] = 0;
  }
}

void clk()
{
  //transfer 
  //Send transfer_value
  digitalWrite(dataOut,transfer_value[clk_counter]);
  //Delay makes reading output signal from next Arduino in chain more reliable.
  delayMicroseconds(16); 
  //Store dataIn value
  transfer_value[clk_counter] = digitalRead(dataIn);
  clk_counter =(clk_counter +1) % CONT_PORT_NUM;
}

void detect_analog()
{
  //LEDは個別なので、ルーチンは変える
  int sensor_state = 0;
  for(int i= 0;i < ANALOG_PORT_NUM;i++)
  {
      //直列に書く
      //LED on
      IR_LED_onoff(i,1);
      //wait
      delayMicroseconds(2000) ;
      //Read Photo Transistor value at LED ON  
      light_on_value[i] = analogRead(i);
      //LED off
      IR_LED_onoff(i,0);
      //wait
      delayMicroseconds(2000) ;
      //Read Photo Transistor value atLED Off  
      light_off_value[i] = analogRead(i);
      diff_value[i] = abs(light_on_value[i] - light_off_value[i]);
      if(diff_value[i] > diff_value_threshold)
      {
        sensor_value[i] = 1;
        sensor_state ++;
      }
  }
  
  //RUN_LEDのところの話
  if(sensor_state != 0)
  {
      digitalWrite(RUN_LED_PIN,HIGH);
  }
  else
  {
      digitalWrite(RUN_LED_PIN,LOW);
  }
}

void IR_LED_onoff(int ch,int on_off)
{
    //1:on 0:off
    digitalWrite(led_pin[ch],on_off);
}


//Old Routin(6ch ver)
/* 
void detect_analog()
{
  if(millis() - gPreviousL5 > SENSOR_INTERVAL)
  {
    switch(reflector_state)
    {
      case 0:
      {
        //LED on
        digitalWrite(IR_LED_PIN,!true);
        reflector_state ++;
        int sensor_state = 0;
        for(int i = 0;i < ANALOG_PORT_NUM ; i++)
        {
          sensor_state += sensor_value[i]; 
        }
        if(sensor_state != 0)
        {
          digitalWrite(RUN_LED_PIN,LOW);
        }
        else
        {
          digitalWrite(RUN_LED_PIN,HIGH);
        }
        break;
      }
      case 1:
      {
        //Read Photo Transistor value 
        for (int i = 0;i < ANALOG_PORT_NUM;i++)
        {
          light_on_value[i] = analogRead(i);
        }
        //LED off
        digitalWrite(IR_LED_PIN,!false);
        reflector_state ++;
        break;
      }
      case 2:
      {
        //LED OFF Wait
        reflector_state ++;
        break;
      }
      case 3:
      {      
        //Read Photo Transistor value 
        for(int i = 0;i < ANALOG_PORT_NUM;i++)
        {
          light_off_value[i] = analogRead(i);
        }
        reflector_state ++;
        break;
      }
      case 19:
      {
        //train existence judgement
        for(int i = 0;i < ANALOG_PORT_NUM;i++)
        {
          diff_value[i] = abs(light_on_value[i] - light_off_value[i]);
          if(diff_value[i] > diff_value_threshold)
          {
            sensor_value[i] = 1;
          }
          Serial.print(diff_value[i]);
          Serial.print(",");
          reflector_state = 0;
        }
        Serial.println("");
        break;
      }
      default:
      {
        reflector_state++;
        break;  
      }
    }
    gPreviousL5 = millis();
  }
} 
*/
